#!/usr/bin/env bash

set -euxo pipefail

tests=(userById spaceById insightById postById notifications lastInsights inspirationPosts createSpace createTask)
repetitions=5 # for correct median selection, keep this value odd, because if even, we would need to take the average of the two middle results
median_index=$((repetitions/2))

# Complex logic to determine the location of the k6 executable.
# Local path for local execution, /tmp path for CI execution in the builder image.
k6_paths=("./k6" "/tmp/k6" "/opt/homebrew/bin/k6" "/home/linuxbrew/.linuxbrew/bin/k6")
find_k6() {
    for path in "${k6_paths[@]}"; do
        if [[ -x "$path" ]]; then
            echo "$path"
            return
        fi
    done
    exit 1
}
k6_path=$(find_k6)

mkdir -p out

output_json_path() {
  echo out/"$1"_"$2".json
}

failures=0

for test in "${tests[@]}"; do
  aggregate_file="$(output_json_path "${test}" aggregate)"
  # reset any residual results from former runs
  echo -n > "$aggregate_file"
  failures_this_test=0
  for i in $(seq 1 ${repetitions}); do
    $k6_path run --env TEST="$test" scripts/performance-measurement.js
    failures_in_run=$(jq '.metrics.custom_failure_count.values.count' "${test}".json)
    if [[ "$failures_in_run" = "0" ]]; then
      # no errors
      jq --sort-keys '.metrics.custom_duration.values += { "repetition": '"$i"' } | .metrics.custom_duration.values' "${test}".json >> "$aggregate_file"
    else
      # request errors
      failures=$((failures+failures_in_run))
      failures_this_test=$((failures_this_test+failures_in_run))
      ((failures++))
      echo "There were $failures_in_run request failures, please inspect the test output."
      echo "Test: ${test} - Repetition: ${i}"
      jq . "${test}".json
    fi
    # save the original for later inspection
    mv "${test}".json "$(output_json_path "${test}" "${i}")"
  done
  # get test timestamp from aggregate file
  if [ "$(uname)" = "Darwin" ]; then
    timestamp=$(stat -f %m "$aggregate_file")
  else
    timestamp=$(stat -c %Y "$aggregate_file")
  fi
  # pick one representative result (selecting the median and the max of the median measurement)
  jq --sort-keys -s 'sort_by(.med) | {"test": "'"${test}"'", "time": '"${timestamp}"', "errors": '"${failures_this_test}"', "median":.['"${median_index}"'], "max":.['"$((repetitions-1))"']}' "$aggregate_file" > "$(output_json_path "${test}" result)"
done

pushd google-cloud-monitoring-reporter
# CI has var defined, but for local execution we need to source .envrc
# shellcheck disable=SC1091
if [[ -z "${GOOGLE_APPLICATION_CREDENTIALS}" ]]; then . .envrc; fi
yarn
yarn report
popd

exit $failures
