import http from 'k6/http'
import { check } from 'k6'
import { sleep } from 'k6'

import { graphQLHeaders, graphQLApiUrl, oryLoginUrl, oryUserIdentifier, oryUserPassword } from './const.js'
import { checkFailureRate, timeToFirstByte } from './main.js'

const MAX_RETRIES = 3  // Max number of retries
const RETRY_DELAY = 2   // Delay between retries

export function loginOryGetAuthHeaders() {
  const oryLoginFlowIdUrl = `${oryLoginUrl}/api`
  const resOryLoginFlowIdUrl = http.get(oryLoginFlowIdUrl)
  const oryLoginFlowId = JSON.parse(resOryLoginFlowIdUrl.body).id

  const loginUrl = `${oryLoginUrl}?flow=${oryLoginFlowId}`
  const resLogin = http.post(loginUrl, {
    identifier: oryUserIdentifier,
    password: oryUserPassword,
    method: 'password',
  }).body
  const resLoginBody = JSON.parse(resLogin)
  return {
    graphQLAuthenticatedUserHeaders: getGraphQLAuthenticatedUserHeaders(resLoginBody.session_token),
    userId: resLoginBody.session.identity.id,
  }
}

// Function to perform batch request with retry logic
export function addBatchRequest(query, variables, tags, headers, retryCount = MAX_RETRIES) {
  const response = http.post(
    graphQLApiUrl,
    JSON.stringify({ query, variables: variables || {} }),
    {
      headers: headers !== undefined ? headers : graphQLHeaders,
      tags,
    }
  )

  // Record the time to first byte (TTFB) for this request
  timeToFirstByte.add(response.timings.waiting, { url: graphQLApiUrl });

  // Check for successful status and retry if needed
  if (check(response, { 'is status 200': (r) => r.status === 200 })) {
    return response
  } else if (retryCount > 0) {
    sleep(RETRY_DELAY)
    console.log(`Request failed. Retrying... [${MAX_RETRIES - retryCount + 1}/${MAX_RETRIES}]`)
    return addBatchRequest(query, variables, tags, headers, retryCount - 1)
  } else {
    console.error(`Request failed after ${MAX_RETRIES} retries.`)
    return response
  }
}

export function createPostRequest(query, variables, tags, headers) {
  return http.post(
    graphQLApiUrl,
    JSON.stringify({
      query,
      variables: variables || {},
    }),
    {
      headers: headers !== undefined ? headers : graphQLHeaders,
      tags,
    },
  )
}

// New function to add batch requests and handle responses
export function addBatchWithRetries(requests) {
  const responses = http.batch(requests);

  responses.forEach((res, index) => {
    // Check and record failure rate
    const isSuccess = check(res, { 'is status 200': (r) => r.status === 200 })
    checkFailureRate.add(!isSuccess, { requestIndex: index })
    
    if (!isSuccess) {
      console.warn(`Request ${index + 1} failed with status ${res.status}`)
    }
  })

  return responses
}


export function checkResponse(response, retryCount = MAX_RETRIES) {
  let checkResponse = check(response, {
    'is status 200': (r) => r.status === 200,
    'has no errors': (r) => JSON.parse(r.body) && JSON.parse(r.body).errors === undefined,
  });

  if (JSON.parse(response.body) && JSON.parse(response.body).errors !== undefined) {
    console.warn('errors in response:', JSON.parse(response.body).errors);
  }
  checkFailureRate.add(!checkResponse);
  timeToFirstByte.add(response.timings.waiting, {
    ttfbURL: response.url,
  })

  // Retry logic
  if (!checkResponse && retryCount > 0) {
    console.log(`Retrying... (${MAX_RETRIES - retryCount + 1}/${MAX_RETRIES})`);
    sleep(RETRY_DELAY); // Wait before retrying

    // Resend the request
    const retryPayload = {
      method: response.request.method,
      url: response.request.url,
      body: response.request.body,
      params: response.request.params,
    };

    const newResponse = http.request(
      retryPayload.method,
      retryPayload.url,
      retryPayload.body,
      retryPayload.params
    );

    // Again with decremented retryCount
    return checkResponse(newResponse, retryCount - 1);
  } else if (!checkResponse) {
    console.error(`Request failed after ${MAX_RETRIES} retries`);
  }

  return checkResponse;
}

// Check multiple responses in a batch
export function checkBatchResponses(responses) {
  for (let i = 0; i < responses.length; i++) {
    checkResponse(responses[i]);
  }
}


export function getGraphQLAuthenticatedUserHeaders(sessionToken) {
  return {
    'X-Disable-Sentry': 'true',
    'Content-Type': 'application/json',
    Authorization: `Bearer ${sessionToken}`,
  }
}

export function isResponseValid(response) {
  return response.status === 200 && JSON.parse(response.body) && JSON.parse(response.body).data
}

export function generateRandomString(prefix = '') {
  return `${prefix} ${(Math.random() + 1).toString(36).substring(2)}`
}
